package com.qmth.sop.server.api;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.sop.business.bean.FormPropertyBean;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.bean.result.FlowResult;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.entity.TFCustomFlow;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.business.entity.TFFlowApprove;
import com.qmth.sop.business.entity.TFFlowLog;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FlowApproveOperationEnum;
import com.qmth.sop.common.enums.FlowApproveSetupEnum;
import com.qmth.sop.common.enums.FlowStatusEnum;
import com.qmth.sop.common.util.GsonUtil;
import com.qmth.sop.common.util.JacksonUtil;
import io.swagger.annotations.*;
import org.activiti.engine.FormService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;
import java.util.zip.ZipInputStream;

@Api(tags = "测试流程表单htmlController")
@RestController
@RequestMapping("/activiti/form/html")
public class ActivitiFromHtmlController {
    private final static Logger log = LoggerFactory.getLogger(ActivitiFromHtmlController.class);

    @Resource
    ActivitiConsumerService activitiConsumerService;

    @Resource
    RepositoryService repositoryService;

    @Resource
    FormService formService;

    @Resource
    TaskService taskService;

    @Resource
    TFCustomFlowService tfCustomFlowService;

    @Resource
    RuntimeService runtimeService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    /**
     * 注册流程
     *
     * @return
     */
    @ApiOperation(value = "创建流程接口")
    @RequestMapping(value = "/createDeployment", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> createDeployment() throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        ClassPathResource resource = new ClassPathResource("testform1.zip");
        ZipInputStream zip = new ZipInputStream(resource.getInputStream());
        builder.addZipInputStream(zip);
        return flowStart(builder.deploy().getId());
    }

    @ApiOperation(value = "获取流程部署信息")
    @RequestMapping(value = "/flow/deployment/data", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowData() {
        return Collections.singletonMap("data", tfCustomFlowService.findFlowDeploymentList());
    }

    @ApiOperation(value = "获取流程提交表单属性接口")
    @RequestMapping(value = "/flow/start", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> flowStart(@ApiParam(value = "部署流程id", required = true) @RequestParam String deploymentId) {
        Map<String, Object> map = new HashMap<>();
        String processDefinitionId = null;
        TFCustomFlow tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getActFlowId, deploymentId));
        Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有部署数据！"));
        if (Objects.isNull(tfCustomFlow.getFlowKey())) {
            processDefinitionId = activitiConsumerService.findProcessDefinitionIdByDeploymentId(deploymentId);
            tfCustomFlow.setFlowKey(processDefinitionId);
            tfCustomFlowService.updateById(tfCustomFlow);
        } else {
            processDefinitionId = tfCustomFlow.getFlowKey();
        }
        FlowResult flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
        Map<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == 1) {
                map.put("setup", flowTaskResultTemp.getSetup());
                map.put("formProperties", flowTaskResultTemp.getFormProperty());
                break;
            }
        }
        map.put("processDefinitionId", processDefinitionId);
        map.put("deploymentId", deploymentId);
        map.put("id", tfCustomFlow.getId());
        return map;
    }

    @ApiOperation(value = "获取流程表单属性接口")
    @RequestMapping(value = "/getFlowFormData", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowFormData(@ApiParam(value = "流程id") @RequestParam(required = false) String flowId,
                                               @ApiParam(value = "流程任务id") @RequestParam(required = false) String taskId) {
//        List<Task> taskList = null;
//        if (Objects.nonNull(flowId) && !Objects.equals(flowId, "")) {
//            taskList = taskService.createTaskQuery().processInstanceId(flowId).list();
//        } else if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {
//            taskList = taskService.createTaskQuery().taskId(taskId).list();
//        }
        Map<String, Object> varMap = new HashMap<>();
//        if (!CollectionUtils.isEmpty(taskList)) {
//            List<FormPropertyBean> formPropertyList = new LinkedList<>();
//            for (Task t : taskList) {
//                String flowTaskId = t.getId();
//                String processInstanceId = t.getProcessInstanceId();
//                Object o = formService.getRenderedTaskForm(flowTaskId);
////                List<FormProperty> formProperties = taskFormData.getFormProperties();
//                log.info("taskId:{},flowId:{}", flowTaskId, processInstanceId);
//                if (Objects.nonNull(o)) {
//                    log.info("task formProperties:{}", JacksonUtil.parseJson(o));
//                    List<FlowFormWidgetResult> list = JSONArray.parseArray(o.toString(), FlowFormWidgetResult.class);
//                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), list));
//                }
//            }
//            varMap.put("formProperties", formPropertyList);
//        }
        return varMap;
    }

    @ApiOperation(value = "启动流程接口")
    @RequestMapping(value = "/startActivityDemo", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> startActivityDemo(@ApiParam(value = "流程定义id", required = true) @RequestParam Long id,
                                                 @ApiParam(value = "审批人id", required = true) @RequestParam String approveId,
                                                 @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile file,
                                                 HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);

        TFCustomFlow tfCustomFlow = tfCustomFlowService.getById(id);
        Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有部署数据！"));
        ProcessInstance processInstance = runtimeService.startProcessInstanceById(tfCustomFlow.getFlowKey(), varMap);
//        ProcessInstance processInstance = formService.submitStartFormData(tfCustomFlow.getFlowKey(), varMap);
        if (Objects.nonNull(processInstance)) {
            TFCustomFlowEntity tfCustomFlowEntity = new TFCustomFlowEntity(tfCustomFlow.getId(), null);
            TFFlowApprove tfFlowApprove = new TFFlowApprove(null, Long.parseLong(approveId), FlowStatusEnum.START, null);
            TFFlowLog tfFlowLog = new TFFlowLog(null, tfFlowApprove.getApproveId(), null, FlowApproveOperationEnum.START, FlowApproveSetupEnum.START.getSetup(), FlowApproveSetupEnum.START.getTitle());
            List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstance.getId()).list();
            if (!CollectionUtils.isEmpty(taskList)) {
                FlowResult flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
                Map<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
                tfCustomFlowEntity.setFlowProcessVar(tfCustomFlow.getFlowProcessVar());
                List<FormPropertyBean> formPropertyList = new LinkedList<>();
                for (Task t : taskList) {
                    String flowTaskId = t.getId();
                    Long flowId = Long.parseLong(t.getProcessInstanceId());
                    varMap.put("flowId", t.getProcessInstanceId());
                    varMap.put("taskId", flowTaskId);
                    tfCustomFlowEntity.setFlowId(flowId);
                    tfFlowApprove.setFlowId(flowId);
                    tfFlowApprove.setApproveMaxSetup(setupMap.size());
                    tfFlowLog.setFlowId(flowId);
                    tfFlowLog.setTaskId(Long.parseLong(flowTaskId));
                    tfFlowLog.setPendApproveId(Arrays.asList(approveId).toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                    log.info("flowTaskId:{},flowId:{}", flowTaskId, flowId);
//                    Object o = formService.getRenderedTaskForm(flowTaskId);
                    FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(t.getTaskDefinitionKey())), FlowTaskResult.class);
                    if (Objects.nonNull(currFlowTaskResult)) {
                        log.info("task formProperties:{}", JacksonUtil.parseJson(currFlowTaskResult));
                        formPropertyList.add(new FormPropertyBean(t.getProcessInstanceId(), flowTaskId, currFlowTaskResult));
                    }
                }
                varMap.put("formProperties", formPropertyList);
            }
            tfCustomFlowEntityService.save(tfCustomFlowEntity);
            tfFlowLog.setObjectId(tfCustomFlowEntity.getId());
            tfFlowApproveService.save(tfFlowApprove);
            tfFlowLogService.save(tfFlowLog);
        }
        return varMap;
    }

    @ApiOperation(value = "获取待办接口")
    @RequestMapping(value = "/getTaskList", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getTaskList(@ApiParam(value = "用户id", required = true) @RequestParam String userId, HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        List<Task> taskList = taskService.createTaskQuery().taskAssignee(userId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                String processDefinitionId = t.getProcessDefinitionId();
                log.info("用户id:{},taskId:{},flowId:{},processDefinitionId:{},formKey:{}", userId, flowTaskId, processInstanceId, processDefinitionId, t.getFormKey());
                if (Objects.nonNull(t.getFormKey())) {
                    Object o = formService.getRenderedTaskForm(flowTaskId);
                    if (Objects.nonNull(o)) {
                        log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                        List<FlowFormWidgetResult> list = JSONArray.parseArray(o.toString(), FlowFormWidgetResult.class);
                        formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, userId, null));
                    }
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    @ApiOperation(value = "流程提交接口")
    @RequestMapping(value = "/complete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> complete(@ApiParam(value = "流程taskId", required = true) @RequestParam String taskId,
                                        @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile[] files, HttpServletRequest request) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);
        String flowTaskId = task.getId();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = task.getProcessDefinitionId();
        formService.submitTaskFormData(flowTaskId, formDataMap);
        List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String nextFlowTaskId = t.getId();
                Object o = formService.getRenderedTaskForm(nextFlowTaskId);
                log.info("userId:{},taskId:{},flowId:{},processDefinitionId:{}", t.getAssignee(), nextFlowTaskId, processInstanceId, processDefinitionId);
                if (Objects.nonNull(o)) {
                    log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                    List<FlowFormWidgetResult> list = JSONArray.parseArray(o.toString(), FlowFormWidgetResult.class);
                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), null));
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    @ApiOperation(value = "结束流程接口")
    @RequestMapping(value = "/deleteProcessInstance", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public boolean deleteProcessInstance(String runId) {
        return activitiConsumerService.deleteProcessInstance(runId);
    }
}